@@ -44,17 +44,17 @@ pub use std::rc;
 // that we have a predictable data representation (since we'll be doing
 // pointer mangling into/out of our heap array.)
 #[repr(C,align(2))]
-struct FirstFitHeap<const HEAPSIZE_WORDS: usize, const SEGS: usize, const S21: usize> {
-  v:  [i16; HEAPSIZE_WORDS],
+struct FirstFitHeap<const SEGS: usize, const S2: usize> {
+  v:  &'static mut [i16],
   pa: [u16; SEGS],
-  st: [i16; S21],
+  st: [i16; S2],
   s:  usize,
   words_per_seg: usize
 }
 
 
-struct GlobalAllocator<const H:usize, const S:usize, const S2:usize> {
-  heap: UnsafeCell<FirstFitHeap<H,S,S2>>
+struct GlobalAllocator<const S:usize, const S2:usize> {
+  heap: UnsafeCell<FirstFitHeap<S,S2>>
 }
 
 // If this is enabled when we do unit tests, then it will replace the default
@@ -65,7 +65,7 @@ struct GlobalAllocator<const H:usize, const S:usize, const S2:usize> {
 //
 #[cfg(target_arch="avr")]
 #[global_allocator]
-static mut GLOBAL: GlobalAllocator::<{memory::alloc::HEAPSIZE/2}, {memory::alloc::SMAX}, {memory::alloc::SMAX * 2}> = GlobalAllocator {
+static mut GLOBAL: GlobalAllocator::<{memory::alloc::SMAX}, {memory::alloc::SMAX * 2}> = GlobalAllocator {
   heap: UnsafeCell::new(FirstFitHeap::new())
 };
 
@@ -73,17 +73,67 @@ static mut GLOBAL: GlobalAllocator::<{memory::alloc::HEAPSIZE/2}, {memory::alloc
 #[cfg(target_arch="avr")]
 #[alloc_error_handler]
 fn _alloc_error(_: core::alloc::Layout) -> ! {
-  panic!();
+  avr_oxide::oserror::halt(avr_oxide::oserror::OsError::OutOfMemory);
 }
 
+/// Allocate memory using the AVRoxide global allocator implementation
+#[cfg(target_arch="avr")]
+pub unsafe fn alloc(layout: Layout) -> *mut u8 {
+  GLOBAL.alloc(layout)
+
+}
+
+/// Allocate memory using the AVRoxide global allocator implementation
+#[cfg(not(target_arch="avr"))]
+pub unsafe fn alloc(layout: Layout) -> *mut u8 {
+  std::alloc::alloc(layout)
+
+}
+
+
+/// Deallocate memory allocated using the AVRoxide global allocator implementation
+#[cfg(target_arch="avr")]
+pub unsafe fn dealloc(ptr: *mut u8, layout: Layout) {
+  GLOBAL.dealloc(ptr, layout)
+}
+
+/// Deallocate memory allocated using the AVRoxide global allocator implementation
+#[cfg(not(target_arch="avr"))]
+pub unsafe fn dealloc(ptr: *mut u8, layout: Layout) {
+  std::alloc::dealloc(ptr, layout)
+}
+
+
 /**
  * Must be called to initialise the global allocator before any allocations
- * are attempted.
+ * are attempted.  We will use all remaining memory on the device (after
+ * the data/BSS segments) as our heap.  We calculate this using the constant
+ * for the current device's memory max, and the `__bss_end` value exported by the
+ * linker.
  */
 #[cfg(target_arch="avr")]
 pub fn initialise() {
   unsafe {
-    let heap = avr_oxide::panic_if_none!(GLOBAL.heap.get().as_mut());
+    let heap = avr_oxide::halt_if_none!(GLOBAL.heap.get().as_mut(), avr_oxide::oserror::OsError::InternalError);
+
+    let mut heap_start_hi:u8;
+    let mut heap_start_lo:u8;
+    core::arch::asm!{
+      "ldi {}, hi8(__heap_start)",
+      "ldi {}, lo8(__heap_start)",
+      out(reg) heap_start_hi,
+      out(reg) heap_start_lo
+    }
+
+    let heap_start = (heap_start_hi as usize) << 8 | heap_start_lo as usize;
+
+    let heap_size = avr_oxide::deviceconsts::memory::SRAM_END as isize - heap_start as isize;
+
+    // Fail if there is not enough memory for a usefully functioning heap
+    if heap_start == 0 || heap_size < avr_oxide::deviceconsts::memory::alloc::MIN_HEAPSIZE as isize {
+      avr_oxide::oserror::halt(avr_oxide::oserror::OsError::NotEnoughRam);
+    }
+    heap.set_heap_location(heap_start as *mut u8, heap_size as usize);
     heap.init();
   }
 }
@@ -92,9 +142,9 @@ pub fn initialise() {
  * Implementation of the Rust GlobalAlloc trait for our First-Fit allocator,
  * allowing it to be used as the Rust global allocator.
  */
-unsafe impl<const H:usize, const S:usize, const S2:usize> GlobalAlloc for GlobalAllocator<H,S,S2> {
+unsafe impl<const S:usize, const S2:usize> GlobalAlloc for GlobalAllocator<S,S2> {
   unsafe fn alloc(&self, layout: Layout) -> *mut u8 {
-    let heap = avr_oxide::panic_if_none!(self.heap.get().as_mut());
+    let heap = avr_oxide::halt_if_none!(self.heap.get().as_mut(), avr_oxide::oserror::OsError::InternalError);
 
     // Calculate the size we need to allocate for the 'worst case' amount of
     // padding needed to achieve the requested alignment
@@ -107,7 +157,7 @@ unsafe impl<const H:usize, const S:usize, const S2:usize> GlobalAlloc for Global
     let size_words=  ((layout.size() + 1) / 2) + align_words;
 
     // Allocate, including space for padding if needed
-    let ptr = avr_oxide::hal::concurrency::interrupt::isolated(||{
+    let ptr = avr_oxide::concurrency::interrupt::isolated(|_isotoken|{
       match heap.bl_new(size_words){
         0 => {
           core::ptr::null_mut()
@@ -147,12 +197,12 @@ unsafe impl<const H:usize, const S:usize, const S2:usize> GlobalAlloc for Global
   }
 
   unsafe fn dealloc(&self, ptr: *mut u8, _layout: Layout) {
-    let heap = avr_oxide::panic_if_none!(self.heap.get().as_mut());
+    let heap = avr_oxide::halt_if_none!(self.heap.get().as_mut(), avr_oxide::oserror::OsError::InternalError);
 
     let index0:usize = core::mem::transmute(&heap.v[0]);
     let mut index = ((ptr as usize) - index0)/2;
 
-    avr_oxide::hal::concurrency::interrupt::isolated(||{
+    avr_oxide::concurrency::interrupt::isolated(|_isotoken|{
       // Check if this allocation actually contains alignment padding,
       // and if so adjust the pointer
       if heap.v[index-1] > 0 {
@@ -166,10 +216,7 @@ unsafe impl<const H:usize, const S:usize, const S2:usize> GlobalAlloc for Global
     });
   }
 }
-
-impl<const HEAPSIZE_WORDS: usize, const SEGS: usize, const S21: usize> FirstFitHeap<HEAPSIZE_WORDS,SEGS,S21> {
-  const HEAPSIZE_MAX: usize = HEAPSIZE_WORDS-1;
-
+impl<const SEGS: usize, const S2: usize> FirstFitHeap<SEGS,S2> {
   /**
    * Create a new instance of the first-fit heap structure.  You must call
    * the init() method before first making any allocations from it.
@@ -179,24 +226,40 @@ impl<const HEAPSIZE_WORDS: usize, const SEGS: usize, const S21: usize> FirstFitH
       // We actually only need to initialise v[0] to reflect the whole block
       // is free - but by copying the value across the whole heap we can use
       // a static allocator
-      v:  [0; HEAPSIZE_WORDS],
+      v:  &mut [],
       pa: [0; SEGS],
-      st: [0; S21],
+      st: [0; S2],
       s: 0,
       words_per_seg: 0
     }
   }
 
+  /**
+   * Set the location of the heap to an address in memory, of the given size
+   * in bytes.  Must be called before `heap.init()`
+   */
+  fn set_heap_location(&mut self, memory: *mut u8, size_bytes: usize){
+    unsafe {
+      self.v = core::slice::from_raw_parts_mut(memory as *mut i16, size_bytes/2)
+    }
+  }
+
   /**
    * Perform such initialisation as is required to make the heap ready for
    * allocations.  Must be called before any calls to `bl_new()` etc.
    */
   fn init(&mut self) {
+    let heapsize_words = self.v.len() as i16;
+
+    if heapsize_words == 0 {
+      avr_oxide::oserror::halt(avr_oxide::oserror::OsError::NotEnoughRam);
+    }
+
     self.s     = 1;
-    self.st[1] = HEAPSIZE_WORDS as i16;
+    self.st[1] = heapsize_words;
     self.pa[0] = 0;
-    self.v[0]  = HEAPSIZE_WORDS as i16;
-    self.words_per_seg = HEAPSIZE_WORDS/SEGS;
+    self.v[0]  = heapsize_words;
+    self.words_per_seg = heapsize_words as usize/SEGS;
 
     self.bl_new(0);
   }
@@ -212,10 +275,16 @@ impl<const HEAPSIZE_WORDS: usize, const SEGS: usize, const S21: usize> FirstFitH
    * Double the number of segments in use.  Assumes the current number is
    * less than (smax/2).
    */
+  #[optimize(speed)]
   fn bl_double(&mut self) {
-    debug_assert!(self.s <= SEGS/2);
+    let heapsize_max = self.v.len() - 1;
+
+    if !(self.s <= SEGS/2) {
+      avr_oxide::oserror::halt(avr_oxide::oserror::OsError::InternalError);
+    }
+
     for i in self.s ..= 2*self.s-1 {
-      self.pa[i] = Self::HEAPSIZE_MAX as u16;
+      self.pa[i] = heapsize_max as u16;
     }
     let mut k = self.s;
 
@@ -237,14 +306,17 @@ impl<const HEAPSIZE_WORDS: usize, const SEGS: usize, const S21: usize> FirstFitH
    * will be allocated or merged with a block on its left in a different
    * segment.
    */
+  #[optimize(speed)]
   fn bl_fix_left(&mut self, addr: usize) {
+    let heapsize_max = self.v.len() - 1;
+
     let mut j = self.bl_seg(addr);
     if (self.v[addr] <= 0) || (self.st[j] as i16 <= self.v[addr]) {
 
       let mut pj = self.pa[j-self.s] as usize; // Index of first block in segment
       let mut pn = (j+1-self.s) * self.words_per_seg; // Start of next segment
-      if pn > Self::HEAPSIZE_MAX { // Bound last segment to total heap length
-        pn = Self::HEAPSIZE_MAX;
+      if pn > heapsize_max { // Bound last segment to total heap length
+        pn = heapsize_max;
       }
       let mut mx = if pj < pn {
         // There is a block starting in this segment
@@ -280,6 +352,9 @@ impl<const HEAPSIZE_WORDS: usize, const SEGS: usize, const S21: usize> FirstFitH
    * is freed or merged with a block on its right, or created by splitting
    * (with addr on the right of the split)
    */
+  // We get weird bugs with this routine if we don't have optimize(speed).
+  // Reasons why are wholly mysterious.
+  #[optimize(speed)]
   fn bl_fix_right(&mut self, addr: usize) {
     let mut j = self.bl_seg(addr);
     // If necessary (and possible) double the number of segments
@@ -303,6 +378,7 @@ impl<const HEAPSIZE_WORDS: usize, const SEGS: usize, const S21: usize> FirstFitH
    * the dummy first block).  The address given must be the index of a
    * control word (as will be the index returned)
    */
+  #[optimize(speed)]
   fn bl_pred(&self, addr: usize) -> usize {
     let mut j = self.bl_seg(addr);
     if self.pa[j-self.s] == addr as u16 {
@@ -331,6 +407,7 @@ impl<const HEAPSIZE_WORDS: usize, const SEGS: usize, const S21: usize> FirstFitH
    * Return the index to a block of at least `size` words, or 0 if there
    * is not enough free space.
    */
+  #[optimize(speed)]
   fn bl_new(&mut self, size: usize) -> usize {
     if self.st[1] <= size as i16 { // Not enough free space
       0
@@ -369,14 +446,16 @@ impl<const HEAPSIZE_WORDS: usize, const SEGS: usize, const S21: usize> FirstFitH
    * Dispose of a block obtained with `bl_new(size)`.  The given address is
    * an index that was returned by `bl_new`.
    */
+  #[optimize(speed)]
   fn bl_dispose(&mut self, addr: usize) {
+    let heapsize_max = self.v.len() - 1;
     let p = addr-1; // Index of control word
     let vp = -self.v[p];
     if vp > 0 {
       self.v[p] = vp; // Mark the block as free
       let j = self.bl_seg(p);
       let pn = p + vp as usize;
-      if pn < Self::HEAPSIZE_MAX && self.v[pn] >= 0 {
+      if pn < heapsize_max && self.v[pn] >= 0 {
         // Next block is empty, so we can merge
         self.v[p] = vp + self.v[pn];
         let jn = self.bl_seg(pn);
@@ -398,7 +477,8 @@ impl<const HEAPSIZE_WORDS: usize, const SEGS: usize, const S21: usize> FirstFitH
         self.bl_fix_right(p);
       }
     } else {
-      panic!(); // Double-dispose is an error
+      // Double-Dispose is an error
+      avr_oxide::oserror::halt(avr_oxide::oserror::OsError::InternalError);
     }
   }
 
@@ -410,7 +490,7 @@ impl<const HEAPSIZE_WORDS: usize, const SEGS: usize, const S21: usize> FirstFitH
     let p = self.v[addr-1];
 
     if p > 0 {
-      panic!();
+      avr_oxide::oserror::halt(avr_oxide::oserror::OsError::InternalError);
     } else {
       -(p+1) as usize
     }
@@ -445,14 +525,18 @@ mod test {
 
   #[test]
   fn test_create_heap() {
-    let mut heap = FirstFitHeap::<512,32,64>::new();
+    let mut memory = [0u8;1024];
+    let mut heap = FirstFitHeap::<32,64>::new();
+    heap.set_heap_location(&mut memory as *mut u8, 1024);
     heap.init();
     assert_eq!(heap.bl_seg(1), 1);
   }
 
   #[test]
   fn test_alloc_free() {
-    let mut heap = FirstFitHeap::<512,32,64>::new();
+    let mut memory = [0u8;1024];
+    let mut heap = FirstFitHeap::<32,64>::new();
+    heap.set_heap_location(&mut memory as *mut u8, 1024);
     heap.init();
 
     let init_free = heap.bl_available();
@@ -555,12 +639,12 @@ mod test {
   #[test]
   pub fn test_global_alloc() {
     unsafe {
+      let mut memory = [0u8;1024];
       let global = GlobalAllocator {
-        heap: UnsafeCell::new(FirstFitHeap::<512,32,64>::new())
+        heap: UnsafeCell::new(FirstFitHeap::<32,64>::new())
       };
-
-      let mut heap = global.heap.get().as_mut().unwrap();
-
+      let heap = global.heap.get().as_mut().unwrap();
+      heap.set_heap_location(&mut memory as *mut u8, 1024);
       heap.init();
 
 
@@ -613,17 +697,41 @@ mod test {
   }
 
   #[test]
-  pub fn test_soaktest_global_alloc() {
+  pub fn soaktest_global_alloc(){
+    // Remember: global alloc allocates 16bit words, not 8-bit bytes,
+    // so the heapsize here is (device heap in `deviceconsts.rs`/2)
+    // ATmega328P small
+    soaktest_global_alloc_sized::<64,8,{8*2}>();  // '328p Small
+    soaktest_global_alloc_sized::<128,8,{8*2}>(); // '328p Medium
+    soaktest_global_alloc_sized::<256,16,{16*2}>(); // '328p Large
+
+    soaktest_global_alloc_sized::<256,32,{32*2}>();  // '4809 Small
+    soaktest_global_alloc_sized::<512,64,{64*2}>(); // '4809 Medium
+    soaktest_global_alloc_sized::<1536,128,{128*2}>(); // '4809 Large
+
+  }
+
+  pub fn soaktest_global_alloc_sized<const SIZE:usize,const SMAX:usize,const S2:usize>() {
+    println!("");
+    println!("==========================");
+    println!("==  ALLOCATOR SOAKTEST  ==");
+    println!("==========================");
+    println!("  Heap:     {} words", SIZE);
+    println!("  Segments: {} (max)", SMAX);
+
+    let mut memory = [0u16; SIZE];
+
+
     unsafe {
       let global = GlobalAllocator {
-        heap: UnsafeCell::new(FirstFitHeap::<512,32,64>::new())
+        heap: UnsafeCell::new(FirstFitHeap::<SMAX,S2>::new())
       };
 
       let mut heap = global.heap.get().as_mut().unwrap();
-
+      heap.set_heap_location(&mut memory as *mut u16 as *mut u8, {SIZE*2});
       heap.init();
 
-      for blocksize in 1..1000 {
+      for blocksize in 1..SMAX {
         println!("Testing blocks of size {}", blocksize);
 
         let mut blocks : Vec<*mut u8> = Vec::new();
