@@ -14,7 +14,7 @@ use avr_oxide::devices::serialport::SerialPort;
 use avr_oxide::devices::wallclock::WallClock;
 use avr_oxide::hal::generic::port::Pin;
 use avr_oxide::oxide::OxideSupervisor;
-use avr_oxide::util::OwnOrBorrowMut;
+use avr_oxide::util::OwnOrBorrow;
 
 pub mod masterclock;
 pub mod wallclock;
@@ -22,18 +22,19 @@ pub mod led;
 pub mod button;
 pub mod serialport;
 pub mod debouncer;
+pub mod inverter;
 
 /// A simple button attached to a hardware pin that generates events with the
 /// standard AVRoxide Supervisor implementation.
-pub type OxideButton<'a,'b,P> = Button<'a,P,OxideSupervisor<'b>>;
+pub type OxideButton<'a,'b> = Button<'a,OxideSupervisor<'b>>;
 /// A simple LED attached to a hardware pin.
-pub type OxideLed<P> = Led<P>;
+pub type OxideLed = Led;
 /// A high-frequency, low-accuracy clock event source that works with the
 /// standard AVRoxide Supervisor implementation.
 pub type OxideMasterClock<'a,'b,T> = MasterClock<'a,T,OxideSupervisor<'b>>;
 /// A simple serial port device that works with the standard AVRoxide
 /// Supervisor implementation.
-pub type OxideSerialPort<'a,'s,P,T,R> = SerialPort<'a,P,T,R,OxideSupervisor<'s>>;
+pub type OxideSerialPort<'a,'s> = SerialPort<'a,OxideSupervisor<'s>>;
 /// A low-frequency, high-accuracy clock event source that works with the
 /// standard AVRoxide Supervisor implementation.
 pub type OxideWallClock<'a,'b,T> = WallClock<'a,T,OxideSupervisor<'b>>;
@@ -41,18 +42,15 @@ pub type OxideWallClock<'a,'b,T> = WallClock<'a,T,OxideSupervisor<'b>>;
 
 /// Convenience trait implemented by any devices that use (and are constructed
 /// with) an I/O pin.
-pub trait UsesPin<P>
-where
-  P: 'static + Pin
-{
+pub trait UsesPin {
   /// Create an instance of this device that uses the given instance of
   /// a pin (by ownership or reference.)
-  fn using<OP: Into<OwnOrBorrowMut<'static, P>>>(pin: OP) -> Self;
+  fn using<OP: Into<OwnOrBorrow<'static, dyn Pin>>>(pin: OP) -> Self;
 
-  /// Convenience method to create an instance of this device with a
-  /// given static reference to a pin (avoids a certain amount of type
-  /// hinting boilerplate for the caller.)
-  fn with_pin(pin: &'static mut P) -> Self
+//  /// Convenience method to create an instance of this device with a
+//  /// given static reference to a pin (avoids a certain amount of type
+//  /// hinting boilerplate for the caller.)
+  fn with_pin(pin: &'static dyn Pin) -> Self
   where
     Self: Sized
   {
@@ -66,10 +64,7 @@ where
   /// This method leaks memory, deliberately - the create instance will never
   /// be freed.  In our intended embedded environment, this is entirely
   /// deliberate.
-  fn static_using<OP: Into<OwnOrBorrowMut<'static, P>>>(pin: OP) -> &'static mut Self
-  where
-    Self: Sized
-  {
+  fn static_using<OP: Into<OwnOrBorrow<'static, dyn Pin>>>(pin: OP) -> &'static mut Self where Self:Sized {
     Box::leak(Box::new(Self::using(pin)))
   }
 
@@ -80,10 +75,7 @@ where
   /// This method leaks memory, deliberately - the create instance will never
   /// be freed.  In our intended embedded environment, this is entirely
   /// deliberate.
-  fn static_with_pin(pin: &'static mut P) -> &'static mut Self
-  where
-    Self: Sized
-  {
+  fn static_with_pin(pin: &'static dyn Pin) -> &'static mut Self where Self:Sized {
     Box::leak(Box::new(Self::with_pin(pin)))
   }
 }
@@ -175,3 +167,18 @@ where
   }
 }
 
+impl<T> Copy for Handle<T>
+where
+  T: avr_oxide::devices::internal::StaticShareable
+{
+}
+
+unsafe impl<T> Send for Handle<T>
+where
+  T: avr_oxide::devices::internal::StaticShareable + Send + 'static,
+{ }
+unsafe impl<T> Sync for Handle<T>
+  where
+    T: avr_oxide::devices::internal::StaticShareable + Sync + 'static,
+{ }
+
