@@ -15,7 +15,11 @@
 pub mod panicout {
   #[doc(hidden)]
   #[macro_export]
-  macro_rules! panic_stdout { () => { avr_oxide::hal::atmega328p::serial::usart0::instance() }}
+  macro_rules! panic_stdout {
+    () => {
+      avr_oxide::hal::atmega328p::serial::usart0::instance()
+    }
+  }
 }
 
 // Code ======================================================================
@@ -25,6 +29,6 @@ pub mod usart0 {
 
   atmel_simple_usart_tpl!(super::super::ADDR_USART,
     avr_oxide::hal::generic::serial::SerialPortIdentity::Usart0,
-    avr_oxide::hal::generic::serial::base::simple::AtmelUsart<16>,
-    _ivr_usart_rcx, _ivr_usart_tcx, _ivr_usart_dre);
+    avr_oxide::hal::generic::serial::base::simple::AtmelUsart<{avr_oxide::deviceconsts::oxide::SERIAL_BUFFER}>,
+    "usart_rcx", "usart_tcx", "usart_dre");
 }
