@@ -17,30 +17,30 @@
 pub mod tcb0 {
   use avr_oxide::atmel_tcb;
 
-  atmel_tcb!(super::super::ADDR_TCB0, avr_oxide::hal::generic::timer::TimerIdentity::Tcb0, _ivr_tcb0_int);
+  atmel_tcb!(super::super::ADDR_TCB0, avr_oxide::hal::generic::timer::TimerIdentity::Tcb0, "tcb0_int");
 }
 #[cfg(feature="tcb1")]
 pub mod tcb1 {
   use avr_oxide::atmel_tcb;
-  atmel_tcb!(super::super::ADDR_TCB1, avr_oxide::hal::generic::timer::TimerIdentity::Tcb1, _ivr_tcb1_int);
+  atmel_tcb!(super::super::ADDR_TCB1, avr_oxide::hal::generic::timer::TimerIdentity::Tcb1, "tcb1_int");
 }
 #[cfg(feature="tcb2")]
 pub mod tcb2 {
   use avr_oxide::atmel_tcb;
 
-  atmel_tcb!(super::super::ADDR_TCB2, avr_oxide::hal::generic::timer::TimerIdentity::Tcb2, _ivr_tcb2_int);
+  atmel_tcb!(super::super::ADDR_TCB2, avr_oxide::hal::generic::timer::TimerIdentity::Tcb2, "tcb2_int");
 }
 #[cfg(feature="tcb3")]
 pub mod tcb3 {
   use avr_oxide::atmel_tcb;
 
-  atmel_tcb!(super::super::ADDR_TCB3, avr_oxide::hal::generic::timer::TimerIdentity::Tcb3, _ivr_tcb3_int);
+  atmel_tcb!(super::super::ADDR_TCB3, avr_oxide::hal::generic::timer::TimerIdentity::Tcb3, "tcb3_int");
 }
 #[cfg(feature="rtc")]
 pub mod rtc {
   use avr_oxide::atmel_rtc;
 
-  atmel_rtc!(super::super::ADDR_RTC, avr_oxide::hal::generic::timer::TimerIdentity::Rtc, _ivr_rtc_pit);
+  atmel_rtc!(super::super::ADDR_RTC, avr_oxide::hal::generic::timer::TimerIdentity::Rtc, "rtc_pit");
 }
 
 
