@@ -8,8 +8,18 @@
 //!
 
 // Imports ===================================================================
+use core::arch::asm;
+use avr_oxide::concurrency::interrupt;
+use avr_oxide::concurrency::util::ThreadId;
+use avr_oxide::util::datatypes::{BitField, BitFieldAccess, BitIndex};
+use avr_oxide::hardware::cpu::cpuregs::{IOREG_CONTEXT_FLAGS,IOREG_SREG};
 
 // Declarations ==============================================================
+pub const CONTEXT_FLAG_INISR: u8 = 3;
+pub const CONTEXT_FLAG_ENABLEINTS: u8 = 2;
+pub const CONTEXT_FLAG_RETI: u8 = 1;
+pub const CONTEXT_FLAG_RESTORED: u8 = 0;
+
 /**
  * Types of protected configuration change
  */
@@ -22,7 +32,47 @@ pub enum ConfigurationChange {
   ProtectedRegister
 }
 
+/**
+ * Thread execution context structure for this CPU.  As used by the
+ * save_thread_context and load_thread_context extern functions.
+ */
+#[repr(C)]
+#[derive(Clone)]
+pub(crate) struct ProcessorContext {
+  pub(crate) sreg: BitField,          // +0
+  pub(crate) gpregs: [u8; 32],           // +1 to +32
+  pub(crate) pc: u16,                    // +33 (pcl) +34 (pch)
+  pub(crate) sp: u16,                    // +35 (spl) +36 (sph)
 
+  pub(crate) tid: ThreadId,              // +37 Thread ID
+
+  #[cfg(feature="extended_addressing")]
+  pub(crate) rampx: u8,
+  #[cfg(feature="extended_addressing")]
+  pub(crate) rampy: u8,
+  #[cfg(feature="extended_addressing")]
+  pub(crate) rampz: u8,
+  #[cfg(feature="extended_addressing")]
+  pub(crate) eind: u8
+}
+
+/**
+ * Handy methods for a CPU implementing savable/restorable context.
+ *
+ * In practice that means `save_thread_context` and `restore_thread_context`
+ * methods implemented in our assembly code section (the somewhat misnamed
+ * `boot.S` that is increasingly about more than booting.)
+ */
+pub(crate) trait ContextSaveRestore {
+  /// Retrieve the current ThreadContext structure as referenced by the
+  /// thread context register.
+  unsafe fn get_processor_context(&self, isotoken: interrupt::token::Isolated) -> &mut ProcessorContext;
+
+  /// Return true if this thread was just restored (as opposed to just being
+  /// saved).
+  fn was_thread_restored(&self) -> bool;
+}
+#[cfg(target_arch="avr")]
 pub trait Cpu {
   /**
    * Write to a protected register.
@@ -31,31 +81,158 @@ pub trait Cpu {
                             register: &mut u8, value: u8 );
 
   /**
-   * Read the current stack pointer
+   * Read the current stack pointer.
+   *
+   * Note that it is critical this function is inlined if it is to read the
+   * SP value you expect :-).
    */
-  fn read_sp(&self) -> u16;
+  #[inline(always)]
+  fn read_sp(&self) -> u16 {
+    let sphval: u8;
+    let splval: u8;
+
+    unsafe {
+      asm!(
+        "in {reg_sphval},{sphregio}",
+        "in {reg_splval},{splregio}",
+        reg_sphval = out(reg) sphval,
+        reg_splval = out(reg) splval,
+        sphregio = const(avr_oxide::hardware::cpu::cpuregs::IOADR_SPH),
+        splregio = const(avr_oxide::hardware::cpu::cpuregs::IOADR_SPL)
+      );
+    }
+
+    ((sphval as u16) << 8) | (splval as u16)
+  }
 
   /**
    * Write the stack pointer.
+   *
+   * Note that it is critical this function is inlined if it is to read the
+   * SP value you expect :-).
    */
-  unsafe fn write_sp(&mut self, sp: u16);
+  #[inline(always)]
+  unsafe fn write_sp(&mut self, sp: u16) {
+    let sphval:u8 = ((sp & 0xff00) >> 8) as u8;
+    let splval:u8 = (sp & 0x00ff) as u8;
+
+    asm!(
+      "out {sphregio},{reg_sphval}",
+      "out {splregio},{reg_splval}",
+      reg_sphval = in(reg) sphval,
+      reg_splval = in(reg) splval,
+      sphregio = const(avr_oxide::hardware::cpu::cpuregs::IOADR_SPH),
+      splregio = const(avr_oxide::hardware::cpu::cpuregs::IOADR_SPL)
+    );
+  }
 
   /**
    * Read the current status register
    */
-  fn read_sreg(&self) -> u8;
+  #[inline(always)]
+  fn read_sreg(&self) -> u8 {
+    unsafe {
+      IOREG_SREG.read_byte()
+    }
+  }
 
   /**
    * Write the status register
    */
-  unsafe fn write_sreg(&mut self, sp: u8);
+  #[inline(always)]
+  fn write_sreg(&mut self, sregval: u8) {
+    unsafe {
+      IOREG_SREG.write_byte(sregval);
+    }
+  }
 
   /**
    * Return true iff interrupts are enabled
    */
-  fn interrupts_enabled(&self) -> bool;
+  #[inline(always)]
+  fn interrupts_enabled(&self) -> bool {
+    unsafe {
+      IOREG_SREG.is_set(BitIndex::bit_c(7))
+    }
+  }
+
+  /**
+   * Return true iff we are inside an interrupt service routine
+   */
+  #[inline(never)]
+  fn in_isr(&self) -> bool {
+    unsafe {
+      IOREG_CONTEXT_FLAGS.is_set_c::<{avr_oxide::hal::generic::cpu::CONTEXT_FLAG_INISR}>()
+    }
+  }
 }
 
+#[cfg(not(target_arch="avr"))]
+pub trait Cpu {
+  /**
+   * Write to a protected register.
+   */
+  unsafe fn write_protected(&mut self, change: ConfigurationChange,
+                            register: &mut u8, value: u8 );
+
+  /**
+   * Read the current stack pointer.
+   *
+   * Note that it is critical this function is inlined if it is to read the
+   * SP value you expect :-).
+   */
+  #[inline(always)]
+  fn read_sp(&self) -> u16 {
+    unimplemented!()
+  }
+
+  /**
+   * Write the stack pointer.
+   *
+   * Note that it is critical this function is inlined if it is to read the
+   * SP value you expect :-).
+   */
+  #[inline(always)]
+  unsafe fn write_sp(&mut self, sp: u16) {
+    unimplemented!()
+  }
+
+  /**
+   * Read the current status register
+   */
+  #[inline(always)]
+  fn read_sreg(&self) -> u8 {
+    unimplemented!()
+  }
+
+  /**
+   * Write the status register
+   */
+  #[inline(always)]
+  unsafe fn write_sreg(&mut self, sregval: u8) {
+    unimplemented!()
+  }
+
+  /**
+   * Return true iff interrupts are enabled
+   */
+  #[inline(always)]
+  fn interrupts_enabled(&self) -> bool {
+    unimplemented!()
+  }
+
+  /**
+   * Return true iff we are inside an interrupt service routine
+   */
+  #[inline(always)]
+  fn in_isr(&self) -> bool {
+    false
+  }
+
+}
+
+
+
 pub trait ClockControl {
   /**
    * Configure the peripheral clock prescaler
@@ -136,8 +313,8 @@ pub trait SleepControl {
 // Code ======================================================================
 #[cfg(target_arch="avr")]
 pub mod base {
-  use avr_oxide::{v_read,v_write};
   use avr_oxide::hal::generic::cpu::{ConfigurationChange, Cpu, ClockControl, SleepControl, private::PermitIdleToken, private::PermitStandbyToken};
+  use avr_oxide::util::datatypes::Volatile;
 
   #[repr(C)]
   pub struct AvrCpuControlBlock {
@@ -166,13 +343,15 @@ pub mod base {
 
   #[repr(C)]
   pub struct AvrSleepControlBlock {
-    pub(crate) ctrla: u8
+    pub(crate) ctrla: Volatile<u8>
   }
 
+
+
   pub struct AvrSleepController {
     pub(crate) scb: &'static mut AvrSleepControlBlock,
-    pub(crate) idle_inhibits: u8,
-    pub(crate) sleep_inhibits: u8
+    pub(crate) idle_inhibits: Volatile<u8>,
+    pub(crate) sleep_inhibits: Volatile<u8>
   }
 
   // Provided by `boot.S`
@@ -191,37 +370,6 @@ pub mod base {
           ccp_io_write(register as *mut u8, value),
       }
     }
-
-    #[inline(always)]
-    fn read_sp(&self) -> u16 {
-      unsafe {
-        v_read!(u16, self.sp)
-      }
-    }
-
-    #[inline(always)]
-    unsafe fn write_sp(&mut self, sp: u16) {
-      v_write!(u16, self.sp, sp)
-    }
-
-    #[inline(always)]
-    fn read_sreg(&self) -> u8 {
-      unsafe {
-        v_read!(u8, self.sreg)
-      }
-    }
-
-    #[inline(always)]
-    unsafe fn write_sreg(&mut self, sreg:u8) {
-      v_write!(u8, self.sreg, sreg);
-    }
-
-    #[inline(always)]
-    fn interrupts_enabled(&self) -> bool {
-      unsafe {
-        (v_read!(u8, self.sreg) & 0b10000000) > 0
-      }
-    }
   }
 
   impl ClockControl for AvrClockControlBlock {
@@ -242,7 +390,7 @@ pub mod base {
           12 => (0x0A << 1) | 0x01,
           24 => (0x0B << 1) | 0x01,
           48 => (0x0C << 1) | 0x01,
-          _ => panic!()
+          _ => avr_oxide::oserror::halt(avr_oxide::oserror::OsError::BadParams)
         };
         ccp_io_write(&mut self.mclkctrlb as *mut u8, pdiv_pen_val);
       }
@@ -251,30 +399,28 @@ pub mod base {
 
   impl AvrSleepController {
     /// Set the hardware `SLPCTRL.ctrla` field according to what is
-    /// currently permitted by our imhibit flags
+    /// currently permitted by our inhibit flags
     fn set_sleep_state(&mut self) {
-      unsafe {
-        v_write!(u8, self.scb.ctrla, match (self.standby_permitted(), self.idle_permitted()) {
-          (true, true)  => 0b00000011, // Standby permitted
-          (false, true) => 0b00000001, // Idle permitted
-          _             => 0b00000000  // No sleep permitted
-        });
-        // Worth noting, if by request Standby is permitted but Idle is
-        // inhibited, we inhibit all sleep (since Standby is Idle++)
-      }
+      self.scb.ctrla.write(match (self.standby_permitted(), self.idle_permitted()) {
+        (true, true)  => 0b00000011, // Standby permitted
+        (false, true) => 0b00000001, // Idle permitted
+        _             => 0b00000000  // No sleep permitted
+      });
+      // Worth noting, if by request Standby is permitted but Idle is
+      // inhibited, we inhibit all sleep (since Standby is Idle++)
     }
   }
 
   impl SleepControl for AvrSleepController {
     unsafe fn reset(&mut self) {
-      self.sleep_inhibits = 0;
-      self.idle_inhibits = 0;
+      self.sleep_inhibits = 0.into();
+      self.idle_inhibits = 0.into();
       self.set_sleep_state();
     }
 
     fn inhibit_standby(&mut self) -> PermitStandbyToken {
       if self.sleep_inhibits == u8::MAX {
-        panic!();
+        avr_oxide::oserror::halt(avr_oxide::oserror::OsError::NestedInhibitTooDeep);
       }
       self.sleep_inhibits += 1;
       self.set_sleep_state();
@@ -283,7 +429,7 @@ pub mod base {
     }
 
     fn permit_standby(&mut self, _token: PermitStandbyToken) {
-      if self.sleep_inhibits > 0 {
+      if self.sleep_inhibits != 0 {
         self.sleep_inhibits -= 1;
       }
       self.set_sleep_state();
@@ -295,7 +441,7 @@ pub mod base {
 
     fn inhibit_idle(&mut self) -> PermitIdleToken {
       if self.idle_inhibits == u8::MAX {
-        panic!();
+        avr_oxide::oserror::halt(avr_oxide::oserror::OsError::NestedInhibitTooDeep);
       }
       self.idle_inhibits += 1;
       self.set_sleep_state();
@@ -304,7 +450,7 @@ pub mod base {
     }
 
     fn permit_idle(&mut self, _token: PermitIdleToken) {
-      if self.idle_inhibits > 0 {
+      if self.idle_inhibits != 0 {
         self.idle_inhibits -= 1;
       }
       self.set_sleep_state();
@@ -318,13 +464,15 @@ pub mod base {
   #[doc(hidden)]
   #[macro_export]
   macro_rules! atmel_cpu_tpl {
-    ($cpuref:expr,$clkref:expr,$slpref:expr) => {
-      use avr_oxide::hal::generic::cpu::base::{AvrCpuControlBlock,AvrClockControlBlock,AvrSleepController};
+    ($cputype:ty, $cpuref:expr,$clkref:expr,$slpref:expr) => {
+      use avr_oxide::hal::generic::cpu::base::{AvrClockControlBlock,AvrSleepController};
       use avr_oxide::mut_singleton;
-      pub type CpuImpl = AvrCpuControlBlock;
+      pub type CpuImpl = $cputype;
       pub type ClockImpl = AvrClockControlBlock;
       pub type SleepImpl = AvrSleepController;
 
+
+
       #[inline(always)]
       pub fn instance() -> &'static mut CpuImpl {
         unsafe {
@@ -340,12 +488,12 @@ pub mod base {
 
       mut_singleton!(
         SleepImpl,
-        SLEEPCTRLINSTANCE,
-        sleepctrl,
+        __SLEEPCTRLINSTANCE,
+        sleepctrl, sleepctrl_isolated,
         AvrSleepController {
           scb: core::mem::transmute($slpref),
-          idle_inhibits: 0,
-          sleep_inhibits: 0
+          idle_inhibits: 0.into(),
+          sleep_inhibits: 0.into()
         });
     }
   }
@@ -353,7 +501,7 @@ pub mod base {
 
 // Dummy Implementations =====================================================
 #[cfg(not(target_arch="avr"))]
-pub mod dummy {
+pub mod base {
   //! Dummy implementation of the CPU interface for running unit tests
   //! of Oxide applications on the developer-environment architecture rather
   //! than AVR.  Unlike the 'true' implementations these use std:: functions
@@ -361,13 +509,15 @@ pub mod dummy {
   use avr_oxide::hal::generic::cpu::{ConfigurationChange, Cpu, ClockControl, SleepControl};
   use avr_oxide::hal::generic::cpu::private::{PermitIdleToken, PermitStandbyToken};
 
-  pub struct DummyCpuControlBlock {
+  pub struct AvrCpuControlBlock {
     pub(crate) sreg: u8
   }
 
   pub struct DummyClockControl {}
 
-  impl Cpu for DummyCpuControlBlock {
+
+
+  impl Cpu for AvrCpuControlBlock {
     unsafe fn write_protected(&mut self, _change: ConfigurationChange, register: &mut u8, value: u8) {
       println!("*** CPU: Protected register write: @{} <- {}", register, value);
     }
@@ -437,21 +587,21 @@ pub mod dummy {
   #[doc(hidden)]
   #[macro_export]
   macro_rules! atmel_cpu_tpl {
-    ($cpuref:expr,$clkref:expr,$sleepref:expr) => {
-      use avr_oxide::hal::generic::cpu::dummy::{DummyCpuControlBlock,DummyClockControl,DummySleepControl};
+    ($cputype:ty, $cpuref:expr,$clkref:expr,$sleepref:expr) => {
+      use avr_oxide::hal::generic::cpu::base::{AvrCpuControlBlock,DummyClockControl,DummySleepControl};
       use avr_oxide::mut_singleton;
 
-      pub type CpuImpl = DummyCpuControlBlock;
+      pub type CpuImpl   = AvrCpuControlBlock;
       pub type ClockImpl = DummyClockControl;
       pub type SleepImpl = DummySleepControl;
 
-      static mut DUMMY_CPU: DummyCpuControlBlock = DummyCpuControlBlock {
+      static mut DUMMY_CPU: AvrCpuControlBlock = AvrCpuControlBlock {
         sreg: 0
       };
       static mut DUMMY_CLOCK: DummyClockControl = DummyClockControl {};
 
       #[inline(always)]
-      pub fn instance() -> &'static mut DummyCpuControlBlock {
+      pub fn instance() -> &'static mut CpuImpl {
         unsafe {
           &mut DUMMY_CPU
         }
@@ -465,8 +615,8 @@ pub mod dummy {
 
       mut_singleton!(
         SleepImpl,
-        SLEEPCTRLINSTANCE,
-        sleepctrl,
+        __SLEEPCTRLINSTANCE,
+        sleepctrl, sleepctrl_isolated,
         DummySleepControl {
         });
 
