@@ -27,21 +27,18 @@
 //! #![no_std]
 //! #![no_main]
 //!
-//! use avr_oxide::hal::atmega4809::hardware;
 //! use avr_oxide::alloc::boxed::Box;
 //! use avr_oxide::devices::UsesPin;
 //! use avr_oxide::devices::debouncer::Debouncer;
 //! use avr_oxide::devices::{ Handle, OxideLed, OxideButton, OxideSerialPort };
-//! use avr_oxide::arduino;
+//! use avr_oxide::boards::board;
 //!
 //!
 //! #[avr_oxide::main(chip="atmega4809")]
 //! pub fn main() {
 //!   let supervisor = avr_oxide::oxide::instance();
-//!   let hardware = hardware::instance();
-//!   let arduino = arduino::nanoevery::Arduino::from(hardware);
 //!
-//!   let mut green_button = Handle::new(OxideButton::using(Debouncer::with_pin(arduino.a2)));
+//!   let mut green_button = Handle::new(OxideButton::using(Debouncer::with_pin(board::pin_a(2))));
 //!   green_button.on_click(Box::new(move |_pinid, _state|{
 //!     // Do some processing when the button is pressed
 //!   }));
@@ -70,21 +67,18 @@
 //! #![no_std]
 //! #![no_main]
 //!
-//! use avr_oxide::hal::atmega4809::hardware;
 //! use avr_oxide::alloc::boxed::Box;
 //! use avr_oxide::devices::UsesPin;
 //! use avr_oxide::devices::debouncer::Debouncer;
 //! use avr_oxide::devices::{ Handle, OxideLed, OxideButton, OxideSerialPort };
-//! use avr_oxide::arduino;
+//! use avr_oxide::boards::board;
 //! use avr_oxide::event::OxideEvent;
 //!
 //! #[avr_oxide::main(chip="atmega4809")]
 //! pub fn main() {
 //!   let supervisor = avr_oxide::oxide::instance();
-//!   let hardware = hardware::instance();
-//!   let arduino = arduino::nanoevery::Arduino::from(hardware);
 //!
-//!   let mut green_button = Handle::new(OxideButton::using(Debouncer::with_pin(arduino.a2)));
+//!   let mut green_button = Handle::new(OxideButton::using(Debouncer::with_pin(board::pin_a(2))));
 //!   green_button.on_click(Box::new(move |_pinid, _state|{
 //!     // Do some processing when the button is pressed
 //!   }));
@@ -98,11 +92,11 @@
 //!       OxideEvent::Initialise => {
 //!         // Do some custom initialisation
 //!         true
-//!       },
+//!       }
 //!       OxideEvent::ClockTick(_timer,_ticks) => {
 //!         // Ignore all clocktick events
 //!         false
-//!       },
+//!       }
 //!       _ => {
 //!         // Everything else should follow default processing
 //!         true
@@ -112,41 +106,41 @@
 //! }
 //! ```
 
+
 // Imports ===================================================================
 use avr_oxide::devices::internal::StaticShareable;
 use avr_oxide::devices::Handle;
 use avr_oxide::private::ringq::RingQ;
 use avr_oxide::event::{OxideEvent, EventSink, EventSource, OxideEventEnvelope};
-use avr_oxide::hal::concurrency::interrupt;
+use avr_oxide::concurrency::Isolated;
+use avr_oxide::deviceconsts::oxide;
+use core::mem::MaybeUninit;
 
 // Declarations ==============================================================
 
 pub struct OxideSupervisor<'e>
 {
-  event_q: RingQ<OxideEventEnvelope<'e>, 16>,
+  event_q: RingQ<OxideEventEnvelope<'e>, {oxide::EVENT_QUEUE}>,
 }
 
-static mut GLOBAL_INSTANCE : Option<&mut OxideSupervisor> = None;
+//static mut GLOBAL_INSTANCE : Option<&mut OxideSupervisor> = None;
+static mut SUPERVISOR : MaybeUninit<OxideSupervisor> = MaybeUninit::uninit();
+
 
 // Code ======================================================================
-pub(crate) fn create_supervisor<'a>() -> OxideSupervisor<'a> {
-  OxideSupervisor {
-    event_q: RingQ::new_with(OxideEventEnvelope::anon(OxideEvent::Initialise)),
-  }
-}
-
-pub(crate) unsafe fn set_global_supervisor(global: &mut OxideSupervisor) {
-  GLOBAL_INSTANCE.replace(core::mem::transmute(global));
+/**
+ * Initialise the global supervisor instance
+ */
+pub(crate) unsafe fn initialise() {
+  core::ptr::write(SUPERVISOR.as_mut_ptr(),
+                   OxideSupervisor {
+                     event_q: RingQ::new_with(OxideEventEnvelope::anon(OxideEvent::Initialise)),
+                   });
 }
 
 pub fn instance() -> &'static mut OxideSupervisor<'static> {
   unsafe {
-    match &mut GLOBAL_INSTANCE {
-      Some(instance) => {
-        *instance
-      },
-      None => panic!()
-    }
+    SUPERVISOR.assume_init_mut()
   }
 }
 
@@ -154,7 +148,6 @@ impl OxideSupervisor<'_> {
   /**
    * Called to have the supervisor listen for events from this device.
    */
-  #[cfg(feature="alloc_ftr")]
   pub fn listen_handle<EP: 'static + EventSource + StaticShareable>(&mut self, source: Handle<EP>) {
     Handle::static_deref(&source).listen();
   }
@@ -176,15 +169,10 @@ impl OxideSupervisor<'_> {
    */
   pub fn run_with_prehandler<F: FnMut(OxideEvent) -> bool>(&mut self, mut pre_handler: F) -> ! {
     loop {
-      interrupt::wait();
+      let event = self.event_q.consume_blocking();
 
-      // We use the no-locking event Q.  This is safe as long as I am
-      // the only consumer, and any producers are running in an interrupt-secure
-      // context.
-      while let Some(event) = self.event_q.consume_nolock() {
-        if pre_handler(event.open_event()) {
-          event.invoke_recipient();
-        }
+      if pre_handler(event.open_event()) {
+        event.invoke_recipient();
       }
     }
   }
@@ -198,13 +186,13 @@ impl OxideSupervisor<'_> {
 }
 
 impl EventSink for OxideSupervisor<'_> {
-  fn event(event: OxideEventEnvelope) {
+  fn event(isotoken: Isolated, event: OxideEventEnvelope) {
+    let supervisor = instance();
+
     unsafe {
-      match &mut GLOBAL_INSTANCE {
-        None => {}
-        Some(supervisor) => {
-          let _ = (*supervisor).event_q.append_nolock(core::mem::transmute(event));
-        }
+      if (*supervisor).event_q.append(isotoken, core::mem::transmute(event)).is_err() {
+        #[cfg(feature="runtime_checks")]
+        avr_oxide::oserror::halt(avr_oxide::oserror::OsError::OxideEventOverflow);
       }
     }
   }
