@@ -26,11 +26,20 @@ pub trait Floored {
   fn floor() -> Self;
 }
 
+#[derive(Copy,Clone,PartialEq,Eq)]
+pub struct DelayQueueHandle(usize);
+
 pub trait DelayQueue<C,T>
 where
   C: Sub + SubAssign + Eq + Ord + Copy + Floored
 {
-  fn insert_at(&mut self, position: C, item: T);
+  fn insert_at(&mut self, position: C, item: T) -> DelayQueueHandle;
+
+  /// Remove an element from the delay queue
+  ///
+  /// Returns `true` if the element was found and removed, `false` if there
+  /// was no matching element.
+  fn remove(&mut self, cancel_handle: DelayQueueHandle) -> bool;
 
   fn decrement(&mut self, by: C);
 
@@ -96,7 +105,7 @@ where
       };
 
       let content = match self.first.take() {
-        None => panic!(),
+        None => avr_oxide::oserror::halt(avr_oxide::oserror::OsError::InternalError),
         Some(node) => {
           self.first = node.next;
           node.contains
@@ -105,13 +114,15 @@ where
       self.ready.push(content);
     }
   }
+
+
 }
 
 impl<C,T> DelayQueue<C,T> for SimpleDelayQueue<C,T>
 where
   C: Sub + SubAssign + Eq + Ord + Copy + Floored
 {
-  fn insert_at(&mut self, mut position: C, item: T) {
+  fn insert_at(&mut self, mut position: C, item: T) -> DelayQueueHandle {
 
     let mut insert_at = &mut self.first as *mut Option<Box<SimpleDelayQueueElement<C,T>>>;
     loop {
@@ -146,10 +157,55 @@ where
         contains: item,
         next: core::mem::replace(&mut *insert_at, Option::None)
       });
+      let dqhandle = DelayQueueHandle(&*new_element as *const SimpleDelayQueueElement<C,T> as usize);
       (&mut *insert_at).replace(new_element);
+
+      dqhandle
     }
   }
 
+  /// Remove an element from the delay queue
+  ///
+  /// Returns `true` if the element was found and removed, `false` if there
+  /// was no matching element.
+  fn remove(&mut self, cancel_handle: DelayQueueHandle) -> bool {
+    let mut candidate = &mut self.first as *mut Option<Box<SimpleDelayQueueElement<C,T>>>;
+    loop {
+      unsafe {
+        match &mut *candidate {
+          None => {
+            // No more candidates to search for - return false
+            return false;
+          },
+          Some(some_element) => {
+            let candidate_handle = DelayQueueHandle(&**some_element as *const SimpleDelayQueueElement<C,T> as usize);
+
+            if candidate_handle == cancel_handle {
+              break;
+            } else {
+              candidate = &mut some_element.next as *mut Option<Box<SimpleDelayQueueElement<C,T>>>;
+            }
+          }
+        }
+      }
+    }
+    // Candidate now points at the Option<> that we are going to remove
+    unsafe {
+      // Extract the contents of the Option, replacing it with None
+      match (&mut *candidate).take() {
+        None => {
+          avr_oxide::oserror::halt(avr_oxide::oserror::OsError::InternalError);
+        },
+        Some(instance) => {
+          // Now replace it with what was in the 'next'
+          let _none = core::mem::replace(&mut *candidate, instance.next);
+        }
+      }
+
+    }
+    true
+  }
+
   #[optimize(speed)]
   fn decrement(&mut self, mut by: C) {
     // Decrement the head element by our count.  It is possible multiple
@@ -304,6 +360,41 @@ mod tests {
     assert_eq!(queue.consume_next_ready(), Option::None);
   }
 
+  #[test]
+  fn delay_queue_remove() {
+    let mut queue = SimpleDelayQueue::<u16,&str>::new();
+
+    queue.insert_at(24, "Fourth");
+    let sixth = queue.insert_at(42, "Sixth");
+    let third = queue.insert_at(17, "Third");
+    let first = queue.insert_at(1, "First");
+    queue.insert_at(16, "Second");
+    queue.insert_at(39, "Fifth");
+
+    // Removing first time should succeed:
+    assert_eq!(queue.remove(first), true);
+    assert_eq!(queue.remove(sixth), true);
+    // Attempting to remove a second time should fail:
+    assert_eq!(queue.remove(sixth), false);
 
 
+    queue.decrement(17);
+    assert_eq!(queue.consume_next_ready(), Option::Some("Third"));
+    assert_eq!(queue.consume_next_ready(), Option::Some("Second"));
+    assert_eq!(queue.consume_next_ready(), Option::None);
+
+    // This should fail to remove, because we already dropped the thing naturally
+    assert_eq!(queue.remove(third), false);
+
+    queue.decrement(17);
+    assert_eq!(queue.consume_next_ready(), Option::Some("Fourth"));
+    assert_eq!(queue.consume_next_ready(), Option::None);
+
+    queue.decrement(17);
+    assert_eq!(queue.consume_next_ready(), Option::Some("Fifth"));
+    assert_eq!(queue.consume_next_ready(), Option::None);
+
+    queue.decrement(17);
+    assert_eq!(queue.consume_next_ready(), Option::None);
+  }
 }
\ No newline at end of file
